package edu.lawrence.hotel;

import java.sql.*;
import java.time.LocalDate;
import java.util.ArrayList;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

/**
 *
 * @author Joe Gregg
 */
public class HotelDAO {

    // The constructor opens the database connection
    public HotelDAO() {
        // Load the database driver
        try {
            Class.forName("com.mysql.cj.jdbc.Driver");
        } catch (ClassNotFoundException ex) {
            System.out.println("Could not load database driver.");
        }
        // Open the connection and set up statements
        try {
            connection = DriverManager.getConnection("jdbc:mysql://localhost:3306/hotel?user=student&password=Cmsc250!");
            statement = connection.createStatement();
            reservationsStmt = connection.prepareStatement(reservationsSQL);
            insertStmt = connection.prepareStatement(insertSQL);
            deleteStmt = connection.prepareStatement(deleteSQL);
        } catch (SQLException ex) {
            System.out.println("Could not connect to database.");
            ex.printStackTrace();
        }

        reservations = FXCollections.observableArrayList();
    }

    public void close() {
        try {
            connection.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
    }

    public ArrayList<String> getRoomNames() {
        ArrayList<String> results = new ArrayList<String>();
        try {
            ResultSet resultSet = statement.executeQuery("select name from rooms");

            while (resultSet.next()) {
                results.add(resultSet.getString(1));
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        return results;
    }

    public void setRoomAndDate(String room, LocalDate date) {
        this.room = room;
        this.date = date;
        reservations.clear();
        try {
            // First look up the capacity of that room
            ResultSet resultSet = statement.executeQuery("select capacity from rooms where name=\"" + room + "\"");
            if (resultSet.next()) {
                capacity = resultSet.getInt(1);
            }

            // Next search for all reservations for that room and that date.
            reservationsStmt.setString(1, room);
            reservationsStmt.setDate(2, java.sql.Date.valueOf(date));
            resultSet = reservationsStmt.executeQuery();
            while (resultSet.next()) {
                Reservation r = new Reservation();
                r.setIdNumber(resultSet.getInt("id"));
                r.setCustomerNumber(resultSet.getInt("customer"));
                r.setGroupSize(resultSet.getInt("size"));
                int start = resultSet.getInt("start");
                r.setStart(start);
                int end = start + resultSet.getInt("duration");
                r.setEnd(end);
                r.setDate(resultSet.getDate("day").toLocalDate());
                reservations.add(r);
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
    }

    public boolean allowsReservation(Reservation newReservation) {
        if (newReservation.getGroupSize() > capacity) {
            return false;
        }
        for (Reservation e : reservations) {
            if (e.compareTo(newReservation) == 0) {
                return false;
            }
        }
        return true;
    }

    public void addReservation(Reservation newReservation) {
        try {
            insertStmt.setInt(1, newReservation.getCustomerNumber());
            insertStmt.setInt(2, newReservation.getGroupSize());
            insertStmt.setDate(3, java.sql.Date.valueOf(date));
            insertStmt.setInt(4, newReservation.getStart());
            insertStmt.setInt(5, newReservation.getEnd() - newReservation.getStart());
            insertStmt.setString(6, room);
            insertStmt.execute();

            String query = "select LAST_INSERT_ID()";
            ResultSet rset = statement.executeQuery(query);
            if (rset.next()) {
                newReservation.setIdNumber(rset.getInt(1));
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        reservations.add(newReservation);
        FXCollections.sort(reservations);
    }

    public void removeReservation(Reservation toRemove) {
        try {
            deleteStmt.setInt(1, toRemove.getIdNumber());
            deleteStmt.execute();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        reservations.remove(toRemove);
    }

    public ObservableList<Reservation> getReservations() {
        return reservations;
    }

    public LocalDate getDate() {
        return date;
    }

    public String getRoom() {
        return room;
    }

    private Connection connection;
    private Statement statement;
    private String reservationsSQL = "select * from reservations where room=? and day=?";
    private PreparedStatement reservationsStmt;
    private String insertSQL = "insert into reservations(customer,size,day,start,duration,room) values(?,?,?,?,?,?)";
    private PreparedStatement insertStmt;
    private String deleteSQL = "delete from reservations where id=?";
    private PreparedStatement deleteStmt;
    private String room;
    private int capacity;
    private LocalDate date;
    private ObservableList<Reservation> reservations;
}
