package edu.lawrence.hotelsql;

import java.net.URL;
import java.time.LocalDate;
import java.util.ResourceBundle;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.ChoiceBox;
import javafx.scene.control.DatePicker;
import javafx.scene.control.ListView;
import edu.lawrence.hotel.HotelDAO;
import edu.lawrence.hotel.Reservation;
import java.util.ArrayList;
import javafx.application.Platform;
import javafx.fxml.FXMLLoader;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Modality;
import javafx.stage.Stage;
import javafx.stage.StageStyle;

/**
 *
 * @author Joe Gregg
 */
public class FXMLDocumentController implements Initializable {
    private HotelDAO dao;
     
    @FXML
    private ListView reservationsList;
    
    @FXML
    private DatePicker datePicker;
    
    @FXML
    private ChoiceBox roomChoice;
    
    @FXML
    private void setDate(ActionEvent event) {
        LocalDate selectedDate = datePicker.getValue();
        String room = roomChoice.getSelectionModel().getSelectedItem().toString();
        dao.setRoomAndDate(room,selectedDate);
        reservationsList.setItems(dao.getReservations());
    }
    
    @FXML
    private void addReservation(ActionEvent event) {
        Stage parent = (Stage) reservationsList.getScene().getWindow();

        try {
            FXMLLoader loader = new FXMLLoader(getClass().getResource("FXMLNewEventDialog.fxml"));
            Parent root = (Parent)loader.load();
            Scene scene = new Scene(root);
            Stage dialog = new Stage();
            dialog.setScene(scene);
            dialog.setTitle("Create New Reservation");
            dialog.initOwner(parent);
            dialog.initModality(Modality.WINDOW_MODAL);
            dialog.initStyle(StageStyle.UTILITY);
            dialog.setX(parent.getX() + parent.getWidth()/4);
            dialog.setY(parent.getY() + parent.getHeight()/3);

            FXMLNewReservationDialogController controller = (FXMLNewReservationDialogController) loader.getController();
            controller.setDAO(dao);
            dialog.show();
        } catch(Exception ex) {
            System.out.println("Could not open dialog.");
            ex.printStackTrace();
        }
    }
    
    @FXML
    private void removeReservation(ActionEvent event) {
        Reservation toRemove = (Reservation) reservationsList.getSelectionModel().getSelectedItem();
        dao.removeReservation(toRemove);
    }
    
    @FXML
    private void exit(ActionEvent event) {
        dao.close();
        Platform.exit();
    }
    
    private void setRoom(Object newRoom) {
        String room = newRoom.toString();
        LocalDate selectedDate = datePicker.getValue();
        dao.setRoomAndDate(room, selectedDate);
        reservationsList.setItems(dao.getReservations());
    }
    
    @Override
    public void initialize(URL url, ResourceBundle rb) {
        dao = new HotelDAO();

        LocalDate selectedDate = LocalDate.now();
        datePicker.setValue(selectedDate);

        ArrayList<String> roomNames = dao.getRoomNames();
        String room = roomNames.get(0);
        roomChoice.getItems().addAll(roomNames);
        roomChoice.setValue(room);
        roomChoice.getSelectionModel().selectedItemProperty().addListener((observable, oldValue, newValue)->setRoom(newValue));

        dao.setRoomAndDate(room, selectedDate);
        reservationsList.setItems(dao.getReservations());
    }    
    
}
